@@ -1,34 +1,5 @@
-# -*- coding: utf-8 -*-
-#
-# Copyright (c) 2011-2020, Cédric Krier
-# Copyright (c) 2013-2021, Nicolas Évrard
-# Copyright (c) 2011-2021, B2CK
-# All rights reserved.
-#
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are met:
-#     * Redistributions of source code must retain the above copyright
-#       notice, this list of conditions and the following disclaimer.
-#     * Redistributions in binary form must reproduce the above copyright
-#       notice, this list of conditions and the following disclaimer in the
-#       documentation and/or other materials provided with the distribution.
-#     * Neither the name of the <organization> nor the
-#       names of its contributors may be used to endorse or promote products
-#       derived from this software without specific prior written permission.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-# ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
-# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
-# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-from __future__ import division
-
+# This file is part of python-sql.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import string
 import numbers
 import warnings
@@ -36,7 +7,7 @@ from threading import local, currentThread
 from collections import defaultdict
 from itertools import chain
 
-__version__ = '1.2.2'
+__version__ = '1.3.0'
 __all__ = ['Flavor', 'Table', 'Values', 'Literal', 'Column', 'Join',
     'Asc', 'Desc', 'NullsFirst', 'NullsLast', 'format2numeric']
 
@@ -282,6 +253,27 @@ class FromItem(object):
     def join(self, right, type_='INNER', condition=None):
         return Join(self, right, type_=type_, condition=condition)
 
+    def left_join(self, right, condition=None):
+        return self.join(right, type_='LEFT', condition=condition)
+
+    def left_outer_join(self, right, condition=None):
+        return self.join(right, type_='LEFT OUTER', condition=condition)
+
+    def right_join(self, right, condition=None):
+        return self.join(right, type_='RIGHT', condition=condition)
+
+    def right_outer_join(self, right, condition=None):
+        return self.join(right, type_='RIGHT OUTER', condition=condition)
+
+    def full_join(self, right, condition=None):
+        return self.join(right, type_='FULL', condition=condition)
+
+    def full_outer_join(self, right, condition=None):
+        return self.join(right, type_='FULL OUTER', condition=condition)
+
+    def cross_join(self, right, condition=None):
+        return self.join(right, type_='CROSS', condition=condition)
+
     def lateral(self):
         return Lateral(self)
 
@@ -801,7 +793,7 @@ class Update(Insert):
     def __str__(self):
         assert all(col.table == self.table for col in self.columns)
         # Get columns without alias
-        columns = map(str, self.columns)
+        columns = list(map(str, self.columns))
 
         with AliasManager():
             from_ = ''
@@ -1425,10 +1417,11 @@ class Collate(Expression):
 
 
 class Window(object):
-    __slots__ = ('_partition', '_order_by', '_frame', '_start', '_end')
+    __slots__ = (
+        '_partition', '_order_by', '_frame', '_start', '_end', '_exclude')
 
     def __init__(self, partition, order_by=None,
-            frame=None, start=None, end=0):
+            frame=None, start=None, end=0, exclude=None):
         super(Window, self).__init__()
         self._partition = None
         self._order_by = None
@@ -1440,6 +1433,7 @@ class Window(object):
         self.frame = frame
         self.start = start
         self.end = end
+        self.exclude = exclude
 
     @property
     def partition(self):
@@ -1469,7 +1463,7 @@ class Window(object):
     @frame.setter
     def frame(self, value):
         if value:
-            assert value in ['RANGE', 'ROWS']
+            assert value in ['RANGE', 'ROWS', 'GROUPS']
         self._frame = value
 
     @property
@@ -1492,6 +1486,16 @@ class Window(object):
             assert isinstance(value, numbers.Integral)
         self._end = value
 
+    @property
+    def exclude(self):
+        return self._exclude
+
+    @exclude.setter
+    def exclude(self, value):
+        if value:
+            assert value in ['CURRENT ROW', 'GROUP', 'TIES']
+        self._exclude = value
+
     @property
     def alias(self):
         return AliasManager.get(self)
@@ -1523,7 +1527,10 @@ class Window(object):
             start = format(self.start, 'PRECEDING')
             end = format(self.end, 'FOLLOWING')
             frame = ' %s BETWEEN %s AND %s' % (self.frame, start, end)
-        return partition + order_by + frame
+        exclude = ''
+        if self.exclude:
+            exclude = ' EXCLUDE %s' % self.exclude
+        return partition + order_by + frame + exclude
 
     @property
     def params(self):
