@@ -1,32 +1,6 @@
-# -*- coding: utf-8 -*-
-#
-# Copyright (c) 2011-2013, Cédric Krier
-# Copyright (c) 2011-2013, B2CK
-# All rights reserved.
-#
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are met:
-#     * Redistributions of source code must retain the above copyright
-#       notice, this list of conditions and the following disclaimer.
-#     * Redistributions in binary form must reproduce the above copyright
-#       notice, this list of conditions and the following disclaimer in the
-#       documentation and/or other materials provided with the distribution.
-#     * Neither the name of the <organization> nor the
-#       names of its contributors may be used to endorse or promote products
-#       derived from this software without specific prior written permission.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-# ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
-# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
-# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-from sql import Expression, Flavor
+# This file is part of python-sql.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
+from sql import Expression, Select, CombiningQuery, Flavor
 
 __all__ = ['Case', 'Coalesce', 'NullIf', 'Greatest', 'Least']
 
@@ -41,6 +15,8 @@ class Conditional(Expression):
     def _format(value):
         if isinstance(value, Expression):
             return str(value)
+        elif isinstance(value, (Select, CombiningQuery)):
+            return '(%s)' % value
         else:
             return Flavor().get().param
 
@@ -66,16 +42,16 @@ class Case(Conditional):
     def params(self):
         p = []
         for cond, result in self.whens:
-            if isinstance(cond, Expression):
+            if isinstance(cond, (Expression, Select, CombiningQuery)):
                 p.extend(cond.params)
             else:
                 p.append(cond)
-            if isinstance(result, Expression):
+            if isinstance(result, (Expression, Select, CombiningQuery)):
                 p.extend(result.params)
             else:
                 p.append(result)
         if self.else_ is not None:
-            if isinstance(self.else_, Expression):
+            if isinstance(self.else_, (Expression, Select, CombiningQuery)):
                 p.extend(self.else_.params)
             else:
                 p.append(self.else_)
@@ -97,7 +73,7 @@ class Coalesce(Conditional):
     def params(self):
         p = []
         for value in self.values:
-            if isinstance(value, Expression):
+            if isinstance(value, (Expression, Select, CombiningQuery)):
                 p.extend(value.params)
             else:
                 p.append(value)
